@@ -26,6 +26,7 @@
 #include <types/obj_type.h>
 #include <types/proxy.h>
 #include <types/stream.h>
+#include <common/buffer.h>
 #include <common/chunk.h>
 #include <common/config.h>
 
@@ -54,6 +55,7 @@ struct appctx {
 	struct applet *applet;     /* applet this context refers to */
 	void *owner;               /* pointer to upper layer's entity (eg: stream interface) */
 	struct act_rule *rule;     /* rule associated with the applet. */
+	struct buffer_wait buffer_wait; /* position in the list of objects waiting for a buffer */
 
 	union {
 		struct {
@@ -100,7 +102,7 @@ struct appctx {
 		struct {
 			unsigned int display_flags;
 			struct pat_ref *ref;
-			struct pat_ref_elt *elt;
+			struct bref bref;	/* back-reference from the pat_ref_elt being dumped */
 			struct pattern_expr *expr;
 			struct chunk chunk;
 		} map;
@@ -114,6 +116,7 @@ struct appctx {
 			struct hlua_socket *socket;
 			struct list wake_on_read;
 			struct list wake_on_write;
+			int die;
 		} hlua;
 		struct {
 			struct hlua hlua;
