@@ -1539,10 +1539,22 @@ int pat_ref_delete_by_id(struct pat_ref *ref, struct pat_ref_elt *refelt)
 {
 	struct pattern_expr *expr;
 	struct pat_ref_elt *elt, *safe;
+	struct bref *bref, *back;
 
 	/* delete pattern from reference */
 	list_for_each_entry_safe(elt, safe, &ref->head, list) {
 		if (elt == refelt) {
+			list_for_each_entry_safe(bref, back, &elt->back_refs, users) {
+				/*
+				 * we have to unlink all watchers. We must not relink them if
+				 * this elt  was the last one in the list.
+				 */
+				LIST_DEL(&bref->users);
+				LIST_INIT(&bref->users);
+				if (elt->list.n != &ref->head)
+					LIST_ADDQ(&LIST_ELEM(elt->list.n, struct stream *, list)->back_refs, &bref->users);
+				bref->ref = elt->list.n;
+			}
 			list_for_each_entry(expr, &ref->pat, list)
 				pattern_delete(expr, elt);
 
@@ -1564,11 +1576,23 @@ int pat_ref_delete(struct pat_ref *ref, const char *key)
 {
 	struct pattern_expr *expr;
 	struct pat_ref_elt *elt, *safe;
+	struct bref *bref, *back;
 	int found = 0;
 
 	/* delete pattern from reference */
 	list_for_each_entry_safe(elt, safe, &ref->head, list) {
 		if (strcmp(key, elt->pattern) == 0) {
+			list_for_each_entry_safe(bref, back, &elt->back_refs, users) {
+				/*
+				 * we have to unlink all watchers. We must not relink them if
+				 * this elt was the last one in the list.
+				 */
+				LIST_DEL(&bref->users);
+				LIST_INIT(&bref->users);
+				if (elt->list.n != &ref->head)
+					LIST_ADDQ(&LIST_ELEM(elt->list.n, struct stream *, list)->back_refs, &bref->users);
+				bref->ref = elt->list.n;
+			}
 			list_for_each_entry(expr, &ref->pat, list)
 				pattern_delete(expr, elt);
 
@@ -1811,6 +1835,7 @@ int pat_ref_append(struct pat_ref *ref, char *pattern, char *sample, int line)
 	else
 		elt->sample = NULL;
 
+	LIST_INIT(&elt->back_refs);
 	LIST_ADDQ(&ref->head, &elt->list);
 
 	return 1;
@@ -1906,6 +1931,7 @@ int pat_ref_add(struct pat_ref *ref,
 	else
 		elt->sample = NULL;
 
+	LIST_INIT(&elt->back_refs);
 	LIST_ADDQ(&ref->head, &elt->list);
 
 	list_for_each_entry(expr, &ref->pat, list) {
@@ -1953,8 +1979,20 @@ void pat_ref_prune(struct pat_ref *ref)
 {
 	struct pat_ref_elt *elt, *safe;
 	struct pattern_expr *expr;
+	struct bref *bref, *back;
 
 	list_for_each_entry_safe(elt, safe, &ref->head, list) {
+		list_for_each_entry_safe(bref, back, &elt->back_refs, users) {
+			/*
+			 * we have to unlink all watchers. We must not relink them if
+			 * this elt  was the last one in the list.
+			 */
+			LIST_DEL(&bref->users);
+			LIST_INIT(&bref->users);
+			if (elt->list.n != &ref->head)
+				LIST_ADDQ(&LIST_ELEM(elt->list.n, struct stream *, list)->back_refs, &bref->users);
+			bref->ref = elt->list.n;
+		}
 		LIST_DEL(&elt->list);
 		free(elt->pattern);
 		free(elt->sample);
@@ -1985,7 +2023,7 @@ struct pattern_expr *pattern_lookup_expr(struct pattern_head *head, struct pat_r
  * flag <reuse> is set.
  */
 struct pattern_expr *pattern_new_expr(struct pattern_head *head, struct pat_ref *ref,
-                                      char **err, int *reuse)
+                                      int patflags, char **err, int *reuse)
 {
 	struct pattern_expr *expr;
 	struct pattern_expr_list *list;
@@ -2008,7 +2046,8 @@ struct pattern_expr *pattern_new_expr(struct pattern_head *head, struct pat_ref
 		list_for_each_entry(expr, &ref->pat, list)
 			if (expr->pat_head->index     == head->index &&
 			    expr->pat_head->parse     == head->parse &&
-			    expr->pat_head->parse_smp == head->parse_smp)
+			    expr->pat_head->parse_smp == head->parse_smp &&
+			    expr->mflags == patflags)
 				break;
 		if (&expr->list == &ref->pat)
 			expr = NULL;
@@ -2029,6 +2068,9 @@ struct pattern_expr *pattern_new_expr(struct pattern_head *head, struct pat_ref
 		/* Initialize this new expr. */
 		pattern_init_expr(expr);
 
+		/* Copy the pattern matching and indexing flags. */
+		expr->mflags = patflags;
+
 		/* This new pattern expression reference one of his heads. */
 		expr->pat_head = head;
 
@@ -2297,10 +2339,9 @@ int pattern_read_from_file(struct pattern_head *head, unsigned int refflags,
 	 */
 	expr = pattern_lookup_expr(head, ref);
 	if (!expr || (expr->mflags != patflags)) {
-		expr = pattern_new_expr(head, ref, err, &reuse);
+		expr = pattern_new_expr(head, ref, patflags, err, &reuse);
 		if (!expr)
 			return 0;
-		expr->mflags = patflags;
 	}
 
 	/* The returned expression may be not empty, because the function
