@@ -886,7 +886,7 @@ int parse_server(const char *file, int linenum, char **args, struct proxy *curpr
 		else if (warnifnotcap(curproxy, PR_CAP_BE, file, linenum, args[0], NULL))
 			err_code |= ERR_ALERT | ERR_FATAL;
 
-		if (!*args[2]) {
+		if (!defsrv && !*args[2]) {
 			Alert("parsing [%s:%d] : '%s' expects <name> and <addr>[:<port>] as arguments.\n",
 			      file, linenum, args[0]);
 			err_code |= ERR_ALERT | ERR_FATAL;
@@ -1048,6 +1048,8 @@ int parse_server(const char *file, int linenum, char **args, struct proxy *curpr
 			newsrv->agent.fall	= curproxy->defsrv.agent.fall;
 			newsrv->agent.health	= newsrv->agent.rise;	/* up, but will fall down at first failure */
 			newsrv->agent.server	= newsrv;
+			if (curproxy->defsrv.resolvers_id != NULL)
+				newsrv->resolvers_id = strdup(curproxy->defsrv.resolvers_id);
 			newsrv->resolver_family_priority = curproxy->defsrv.resolver_family_priority;
 			if (newsrv->resolver_family_priority == AF_UNSPEC)
 				newsrv->resolver_family_priority = AF_INET6;
@@ -1097,6 +1099,7 @@ int parse_server(const char *file, int linenum, char **args, struct proxy *curpr
 				cur_arg += 2;
 			}
 			else if (!strcmp(args[cur_arg], "resolvers")) {
+				free(newsrv->resolvers_id);
 				newsrv->resolvers_id = strdup(args[cur_arg + 1]);
 				cur_arg += 2;
 			}
@@ -2507,10 +2510,12 @@ void apply_server_state(void)
 			else if (diff & PR_FBM_MISMATCH_ID) {
 				Warning("In backend '%s' (id: '%d'): server ID mismatch: from server state file: '%s', from running config %d\n", bk->id, bk->uuid, params[2], srv->puid);
 				send_log(bk, LOG_NOTICE, "In backend '%s' (id: %d): server ID mismatch: from server state file: '%s', from running config %d\n", bk->id, bk->uuid, params[2], srv->puid);
+				continue;
 			}
 			else if (diff & PR_FBM_MISMATCH_NAME) {
 				Warning("In backend '%s' (id: %d): server name mismatch: from server state file: '%s', from running config '%s'\n", bk->id, bk->uuid, params[3], srv->id);
 				send_log(bk, LOG_NOTICE, "In backend '%s' (id: %d): server name mismatch: from server state file: '%s', from running config '%s'\n", bk->id, bk->uuid, params[3], srv->id);
+				continue;
 			}
 
 			/* now we can proceed with server's state update */
@@ -2653,6 +2658,9 @@ int snr_resolution_cb(struct dns_resolution *resolution, struct dns_nameserver *
 			serverip = &((struct sockaddr_in6 *)&s->addr)->sin6_addr.s6_addr;
 			break;
 
+		case AF_UNSPEC:
+			break;
+
 		default:
 			goto invalid;
 	}
