@@ -203,6 +203,11 @@ static void stream_int_shutw(struct stream_interface *si)
 	oc->wex = TICK_ETERNITY;
 	si->flags &= ~SI_FL_WAIT_DATA;
 
+	if (tick_isset(si->hcto)) {
+		ic->rto = si->hcto;
+		ic->rex = tick_add(now_ms, ic->rto);
+	}
+
 	switch (si->state) {
 	case SI_ST_EST:
 		/* we have to shut before closing, otherwise some short messages
@@ -453,9 +458,13 @@ void stream_int_notify(struct stream_interface *si)
 		oc->wex = TICK_ETERNITY;
 	}
 
-	/* indicate that we may be waiting for data from the output channel */
+	/* indicate that we may be waiting for data from the output channel or
+	 * we're about to close and can't expect more data if SHUTW_NOW is there.
+	 */
 	if ((oc->flags & (CF_SHUTW|CF_SHUTW_NOW)) == 0 && channel_may_recv(oc))
 		si->flags |= SI_FL_WAIT_DATA;
+	else if ((oc->flags & (CF_SHUTW|CF_SHUTW_NOW)) == CF_SHUTW_NOW)
+		si->flags &= ~SI_FL_WAIT_DATA;
 
 	/* update OC timeouts and wake the other side up if it's waiting for room */
 	if (oc->flags & CF_WRITE_ACTIVITY) {
@@ -485,6 +494,10 @@ void stream_int_notify(struct stream_interface *si)
 	 * the buffer is full. We must not stop based on input data alone because
 	 * an HTTP parser might need more data to complete the parsing.
 	 */
+
+	/* ensure it's only set if a write attempt has succeeded */
+	ic->flags &= ~CF_WRITE_PARTIAL;
+
 	if (!channel_is_empty(ic) &&
 	    (si_opposite(si)->flags & SI_FL_WAIT_DATA) &&
 	    (ic->buf->i == 0 || ic->pipe)) {
@@ -536,8 +549,6 @@ void stream_int_notify(struct stream_interface *si)
 	}
 	if (ic->flags & CF_READ_ACTIVITY)
 		ic->flags &= ~CF_READ_DONTWAIT;
-
-	stream_release_buffers(si_strm(si));
 }
 
 
@@ -559,7 +570,8 @@ static int si_conn_wake_cb(struct connection *conn)
 	if (conn->flags & CO_FL_ERROR)
 		si->flags |= SI_FL_ERR;
 
-	if (unlikely(!(conn->flags & (CO_FL_WAIT_L4_CONN | CO_FL_WAIT_L6_CONN | CO_FL_CONNECTED)))) {
+	if ((si->state < SI_ST_EST) &&
+	    (conn->flags & (CO_FL_CONNECTED | CO_FL_HANDSHAKE)) == CO_FL_CONNECTED) {
 		si->exp = TICK_ETERNITY;
 		oc->flags |= CF_WRITE_NULL;
 	}
@@ -569,6 +581,7 @@ static int si_conn_wake_cb(struct connection *conn)
 	 * stream-int status.
 	 */
 	stream_int_notify(si);
+	channel_release_buffer(ic, &(si_strm(si)->buffer_wait));
 
 	/* Third step : update the connection's polling status based on what
 	 * was done above (eg: maybe some buffers got emptied).
@@ -599,6 +612,9 @@ static void si_conn_send(struct connection *conn)
 	struct channel *oc = si_oc(si);
 	int ret;
 
+	/* ensure it's only set if a write attempt has succeeded */
+	oc->flags &= ~CF_WRITE_PARTIAL;
+
 	if (oc->pipe && conn->xprt->snd_pipe) {
 		ret = conn->xprt->snd_pipe(conn, oc->pipe);
 		if (ret > 0)
@@ -820,6 +836,11 @@ static void stream_int_shutw_conn(struct stream_interface *si)
 	oc->wex = TICK_ETERNITY;
 	si->flags &= ~SI_FL_WAIT_DATA;
 
+	if (tick_isset(si->hcto)) {
+		ic->rto = si->hcto;
+		ic->rex = tick_add(now_ms, ic->rto);
+	}
+
 	switch (si->state) {
 	case SI_ST_EST:
 		/* we have to shut before closing, otherwise some short messages
@@ -921,6 +942,9 @@ static void stream_int_chk_snd_conn(struct stream_interface *si)
 	struct channel *oc = si_oc(si);
 	struct connection *conn = __objt_conn(si->end);
 
+	/* ensure it's only set if a write attempt has succeeded */
+	oc->flags &= ~CF_WRITE_PARTIAL;
+
 	if (unlikely(si->state > SI_ST_EST || (oc->flags & CF_SHUTW)))
 		return;
 
@@ -1044,14 +1068,14 @@ static void si_conn_recv_cb(struct connection *conn)
 	if (conn->flags & CO_FL_ERROR)
 		return;
 
-	/* stop here if we reached the end of data */
-	if (conn_data_read0_pending(conn))
-		goto out_shutdown_r;
-
 	/* maybe we were called immediately after an asynchronous shutr */
 	if (ic->flags & CF_SHUTR)
 		return;
 
+	/* stop here if we reached the end of data */
+	if (conn_data_read0_pending(conn))
+		goto out_shutdown_r;
+
 	cur_read = 0;
 
 	if ((ic->flags & (CF_STREAMER | CF_STREAMER_FAST)) && !ic->buf->o &&
@@ -1126,8 +1150,8 @@ static void si_conn_recv_cb(struct connection *conn)
 		ic->pipe = NULL;
 	}
 
-	/* now we'll need a buffer */
-	if (!stream_alloc_recv_buffer(ic)) {
+	/* now we'll need a input buffer for the stream */
+	if (!channel_alloc_buffer(ic, &(si_strm(si)->buffer_wait))) {
 		si->flags |= SI_FL_WAIT_ROOM;
 		goto end_recv;
 	}
@@ -1137,7 +1161,7 @@ static void si_conn_recv_cb(struct connection *conn)
 	 * that if such an event is not handled above in splice, it will be handled here by
 	 * recv().
 	 */
-	while (!(conn->flags & (CO_FL_ERROR | CO_FL_SOCK_RD_SH | CO_FL_DATA_RD_SH | CO_FL_WAIT_ROOM | CO_FL_HANDSHAKE))) {
+	while (!(conn->flags & (CO_FL_ERROR | CO_FL_SOCK_RD_SH | CO_FL_WAIT_ROOM | CO_FL_HANDSHAKE)) && !(ic->flags & CF_SHUTR)) {
 		max = channel_recv_max(ic);
 
 		if (!max) {
@@ -1251,7 +1275,6 @@ static void si_conn_recv_cb(struct connection *conn)
 	if (ic->flags & CF_AUTO_CLOSE)
 		channel_shutw_now(ic);
 	stream_sock_read0(si);
-	conn_data_read0(conn);
 	return;
 }
 
@@ -1435,6 +1458,11 @@ static void stream_int_shutw_applet(struct stream_interface *si)
 	oc->wex = TICK_ETERNITY;
 	si->flags &= ~SI_FL_WAIT_DATA;
 
+	if (tick_isset(si->hcto)) {
+		ic->rto = si->hcto;
+		ic->rex = tick_add(now_ms, ic->rto);
+	}
+
 	/* on shutw we always wake the applet up */
 	appctx_wakeup(si_appctx(si));
 
